package info.textgrid.middleware;

import java.util.ArrayList;
import java.util.List;
import info.textgrid.middleware.oaipmh.ListIdentifiersType;
import info.textgrid.middleware.oaipmh.RequestType;

/**
 *
 */
public abstract class IdentifierListDelivererAbstract implements IdentifierListDelivererInterface {

  protected boolean textgrid;
  protected boolean dariah;

  /**
   * @param textgrid
   * @param dariah
   */
  public IdentifierListDelivererAbstract(boolean textgrid, boolean dariah) {
    this.textgrid = textgrid;
    this.dariah = dariah;
  }

  /*
   * (non-Javadoc)
   * 
   * @see info.textgrid.middleware.IdentifierListDelivererInterface#processIdentifierList(java.lang.
   * String, java.lang.String, java.lang.String, java.lang.String)
   */
  @Override
  public ListIdentifiersType processIdentifierList(String from, String to, String set,
      String resumptionToken) {
    return null;
  }

  /**
   * @param request
   * @return
   */
  public static ErrorHandler requestChecker(RequestType request) {

    ErrorHandler result = new ErrorHandler();

    // Check if metadata prefix is existing and valid.
    if (request.getMetadataPrefix() != null
        && !request.getMetadataPrefix().equals(TGConstants.METADATA_DC_PREFIX)
        && !request.getMetadataPrefix().equals(TGConstants.METADATA_IDIOM_PREFIX)) {
      result.setError(TGConstants.OAI_METADATA_FORMAT_ERROR,
          "The value of the metadataPrefix: " + request.getMetadataPrefix()
              + " is not supported by the item identified by the value of: "
              + request.getIdentifier());
    }

    // Check if resumptionToken is invalid or existing.
    if (request.getResumptionToken() != null) {
      boolean restokDCExisting = RecordListDelivererIDIOM.cursorCollector != null
          && RecordListDelivererIDIOM.cursorCollector.containsKey(request.getResumptionToken());
      boolean restokIDIOMExisting = RecordListDelivererDC.cursorCollector != null
          && RecordListDelivererDC.cursorCollector.containsKey(request.getResumptionToken());
      if (restokDCExisting || restokIDIOMExisting) {
        result.setError(TGConstants.OAI_BAD_RESUMPTION_TOKEN, "The value of the "
            + request.getResumptionToken() + " argument is invalid or expired.");
      }
    }

    List<String> errorValues = new ArrayList<String>();

    if (request.getResumptionToken() != null && request.getMetadataPrefix() != null) {
      errorValues.add("metadataPrefix");
    }
    if (request.getResumptionToken() == null && request.getMetadataPrefix() == null) {
      errorValues.add("metadataPrefix");
    }
    if (request.getIdentifier() != null) {
      errorValues.add("identifier");
    }
    if (errorValues.size() > 0) {
      result.setError(TGConstants.OAI_BAD_ARGUMENT, "The request includes illegal arguments "
          + "or is missing required arguments: " + errorValues);
    }

    return result;
  }

}
